import os
import shutil
from uuid import uuid4

import naas_abi_cli

from ..utils.Copier import Copier

LOCAL_ENV_MARKER = "# Added by abi deploy local command execution"

DEFAULT_ENV_VALUES: dict[str, str] = {
    "POSTGRES_USER": "abi",
    "POSTGRES_DB": "abi",
    "POSTGRES_HOST": "postgres",
    "POSTGRES_PORT": "5432",
    "QDRANT_HOST": "qdrant",
    "QDRANT_PORT": "6333",
    "REDIS_PORT": "6379",
    "MINIO_HOST": "minio",
    "MINIO_PORT": "9000",
    "MINIO_ROOT_USER": "abi",
    "ABI_HOST": "abi",
    "PUBLIC_WEB_HOST": "localhost",
    "PUBLIC_API_HOST": "api.localhost",
    "RABBITMQ_USER": "abi",
    "NEXUS_WEB_IMAGE": "ghcr.io/jupyter-naas/nexus-web",
    "NEXUS_WEB_TAG": "latest",
    "NEXUS_WEB_PORT": "3042",
}

RANDOM_ENV_KEYS: tuple[str, ...] = (
    "POSTGRES_PASSWORD",
    "MINIO_ROOT_PASSWORD",
    "RABBITMQ_PASSWORD",
    "FUSEKI_ADMIN_PASSWORD",
)


def _ensure_env_var(env_path: str, key: str, value: str) -> None:
    existing_content = ""
    if os.path.exists(env_path):
        with open(env_path, "r", encoding="utf-8") as env_file:
            existing_content = env_file.read()

    for line in existing_content.splitlines():
        if line.strip().startswith(f"{key}="):
            return

    with open(env_path, "a", encoding="utf-8") as env_file:
        if existing_content and not existing_content.endswith("\n"):
            env_file.write("\n")
        env_file.write(f"{key}={value}\n")


def _append_local_env_once(source_env_path: str, destination_env_path: str) -> None:
    with open(source_env_path, "r", encoding="utf-8") as source_file:
        source_content = source_file.read()

    if source_content.strip() == "":
        open(destination_env_path, "a", encoding="utf-8").close()
        return

    current_content = ""
    if os.path.exists(destination_env_path):
        with open(destination_env_path, "r", encoding="utf-8") as destination_file:
            current_content = destination_file.read()

    if LOCAL_ENV_MARKER in current_content:
        return

    with open(destination_env_path, "a", encoding="utf-8") as destination_file:
        if current_content and not current_content.endswith("\n"):
            destination_file.write("\n")
        destination_file.write(source_content)


def setup_local_deploy(project_path: str) -> None:
    project_path = os.path.abspath(os.path.expanduser(project_path))

    deploy_path = os.path.join(project_path, ".deploy")
    docker_compose_template_path = os.path.join(deploy_path, "docker-compose.yml")
    docker_compose_target_path = os.path.join(project_path, "docker-compose.yml")
    local_env_template_path = os.path.join(deploy_path, ".env")
    local_env_target_path = os.path.join(project_path, ".env")

    if not os.path.exists(deploy_path):
        os.makedirs(deploy_path, exist_ok=False)

        copier = Copier(
            templates_path=os.path.join(
                os.path.dirname(naas_abi_cli.__file__), "cli/deploy/templates/local"
            ),
            destination_path=deploy_path,
        )
        copier.copy(
            values={
                "POSTGRES_USER": "abi",
                "POSTGRES_PASSWORD": str(uuid4()),
                "POSTGRES_DB": "abi",
                "MINIO_ROOT_PASSWORD": str(uuid4()),
                "RABBITMQ_PASSWORD": str(uuid4()),
                "FUSEKI_ADMIN_PASSWORD": str(uuid4()),
            }
        )

    if os.path.exists(docker_compose_template_path):
        if os.path.exists(docker_compose_target_path):
            os.remove(docker_compose_template_path)
        else:
            shutil.move(docker_compose_template_path, docker_compose_target_path)

    if os.path.exists(local_env_template_path):
        _append_local_env_once(local_env_template_path, local_env_target_path)
        os.remove(local_env_template_path)

    for key, value in DEFAULT_ENV_VALUES.items():
        _ensure_env_var(local_env_target_path, key, value)

    for key in RANDOM_ENV_KEYS:
        _ensure_env_var(local_env_target_path, key, str(uuid4()))



import importlib
from naas_abi_cli.cli.stack_runtime import ComposeServiceState
from naas_abi_cli.cli.stack_services import ReadinessResult


stack_module = importlib.import_module("naas_abi_cli.cli.stack")


def test_is_container_in_error_detects_unhealthy_container() -> None:
    state = ComposeServiceState(
        service="abi",
        container_name="abi-1",
        state="running",
        health="unhealthy",
        exit_code=None,
        status="Up",
    )

    assert stack_module._is_container_in_error("abi", state) is True


def test_is_container_in_error_ignores_successful_one_shot_exit() -> None:
    state = ComposeServiceState(
        service="minio-init",
        container_name="minio-init-1",
        state="exited",
        health=None,
        exit_code=0,
        status="Exited (0)",
    )

    assert stack_module._is_container_in_error("minio-init", state) is False


def test_wait_for_stack_readiness_requires_abi_health(monkeypatch) -> None:
    states = {
        "abi": ComposeServiceState(
            service="abi",
            container_name="abi-1",
            state="running",
            health="healthy",
            exit_code=None,
            status="Up",
        ),
        "service-portal": ComposeServiceState(
            service="service-portal",
            container_name="service-portal-1",
            state="running",
            health=None,
            exit_code=None,
            status="Up",
        ),
    }

    monkeypatch.setattr(
        stack_module, "compose_service_list", lambda: ["abi", "service-portal"]
    )
    monkeypatch.setattr(stack_module, "compose_service_states", lambda: states)
    monkeypatch.setattr(
        stack_module,
        "evaluate_service_readiness",
        lambda service_name, state: ReadinessResult(True, "compose", "ready"),
    )

    ready, error_services = stack_module._wait_for_stack_readiness(
        timeout_seconds=1,
        poll_interval_seconds=0,
    )
    assert ready is True
    assert error_services == []


def test_wait_for_stack_readiness_returns_error_services(monkeypatch) -> None:
    states = {
        "abi": ComposeServiceState(
            service="abi",
            container_name="abi-1",
            state="running",
            health="healthy",
            exit_code=None,
            status="Up",
        ),
        "postgres": ComposeServiceState(
            service="postgres",
            container_name="postgres-1",
            state="exited",
            health=None,
            exit_code=1,
            status="Exited (1)",
        ),
    }

    monkeypatch.setattr(
        stack_module, "compose_service_list", lambda: ["abi", "postgres"]
    )
    monkeypatch.setattr(stack_module, "compose_service_states", lambda: states)

    ready, error_services = stack_module._wait_for_stack_readiness(
        timeout_seconds=1,
        poll_interval_seconds=0,
    )
    assert ready is False
    assert error_services == ["postgres"]
